""" Update build status information from CodeBuild EventBridge events
"""

import os

import boto3


def lambda_handler(event, context):
    if (event["detail-type"] == "CodeBuild Build State Change"
            and event["detail"]["build-status"] in ["SUCCEEDED", "FAILED"]):
        set_build_status(
            event["detail"]["project-name"],
            event["detail"]["build-status"]
        )


def set_build_status(build_project, build_status):
    dynamodb_client = boto3.client("dynamodb")
    
    dynamodb_client.put_item(
        TableName=os.environ["DYNAMODB_TABLE"],
        Item={
            "build_project": {"S": build_project},
            "build_status": {"S": build_status}
        }
    )




"""Module that contains shared functions to the test classes
"""

import os
from os.path import join, abspath, dirname, isdir
import shutil

# Consts to be used by the test classes
APP = "update-conf-py-do-not-use"
TESTS_DIR = abspath(dirname(__file__))
SNIPPETS_DIR = join(TESTS_DIR, "snippets")
TMP_DIR = join(TESTS_DIR, "tmp")
CONF_FILE = join(TESTS_DIR, "config", "update-conf-py-do-not-use.conf")
USER_CONF_FILE = join(
        TESTS_DIR, "config", "user-update-conf-py-do-not-use.conf")
RESULTS_DIR = join(TESTS_DIR, "expected_results")
ROOT_DIR = join(TESTS_DIR, os.pardir)


def clean_tmp():
    """Clean the TMP_DIR directory
    """
    if isdir(TMP_DIR):
        shutil.rmtree(TMP_DIR)
    os.mkdir(TMP_DIR)



""" Return CodeBuild build status information for API integration with
shields.io

It works standalone (with Lambda HTTP endpoints) or via integration with
API Gateway.
"""

import json
import os

import boto3
import requests


# Constants
CODEBUILD_DEFAULT_BRANCH = "main"
DEFAULT_LABEL = "build"
CODEBUILD_BADGE_PATH = "/badges"


def lambda_handler(event, context):
    # Validate requests from Lambda HTTPS endpoint
    if "http" in event["requestContext"]:
        if event["requestContext"]["http"]["method"] != "GET":
            return { "statusCode": 405 }
        if event["requestContext"]["http"]["path"] != "/build-status":
            return { "statusCode": 404 }
        if not "uuid" in event["queryStringParameters"]:
            return { 
                "statusCode": 400,
                "body": json.dumps({
                    "message": "Missing required request parameters: [uuid]"
                })
            }
    # Get parameters for Lambda HTTPS endpoint or API Gateway 
    # integration
    uuid = event["queryStringParameters"]["uuid"]
    branch = event["queryStringParameters"].get("branch", CODEBUILD_DEFAULT_BRANCH)
    tag = event["queryStringParameters"].get("tag", None)
    label = event["queryStringParameters"].get("label", DEFAULT_LABEL)

    return {
        "statusCode": 200,
        "body": get_shields_io_json(uuid, branch, tag, label)
    }


def get_shields_io_json(uuid, branch=None, tag=None, label=DEFAULT_LABEL):
    dynamodb_client = boto3.client("dynamodb")
    codebuild_client = boto3.client("codebuild")

    # Set default branch if necessary
    if not (branch or tag):
        branch = CODEBUILD_DEFAULT_BRANCH

    # Check if uuid exist (from cache)
    try:
        dynamodb_client.get_item(
            TableName=os.environ["DYNAMODB_TABLE"],
            Key={"uuid": {"S": uuid}}
        )["Item"]
        uuid_exists = True
    except KeyError:
        uuid_exists = False

    # Get build status
    build_status = "unknown"
    if uuid_exists:
        url = "{}{}".format(codebuild_client.meta.endpoint_url, CODEBUILD_BADGE_PATH)
        # Send request to CodeBuild badge endpoint
        params = { "uuid": uuid }
        if branch:
            params["branch"] = branch
        if tag:
            params["tag"] = tag
        response = requests.get(url, params=params, allow_redirects=False)
        # Process response
        if not response.is_redirect:
            raise RuntimeError("Unexpected status code: {}".format(response.status_code))
        location = response.headers["Location"]
        build_status = location.split("/")[-1].split(".")[0]

    # Map build status to message/color
    if build_status == "passing":
        color = "success"
    elif build_status == "failing":
        color = "critical"
    elif build_status == "unknown":
        build_status = "no status"
        color = "inactive"
    else:
        raise RuntimeError("Not expected `build_status`: {}".format(build_status))
    
    # Return JSON in the format expected by shields.io
    # See: https://shields.io/endpoint
    return json.dumps({
        "schemaVersion": 1,
        "label": label,
        "message": build_status,
        "color": color
    }, indent=4)




import os
from os.path import join, isfile
import shutil
import tempfile
import filecmp

import unittest

from update_conf_py_do_not_use import main
from . import utils


class TempToFileTest(unittest.TestCase):
    """Tests of moving a config temp file to its final location
    """

    def setUp(self):
        utils.clean_tmp()

    def test_temp_to_file(self):
        """Moves a temp file created to its final place
        """
        config_file = join(utils.TMP_DIR, "test1")
        temp_file = join(tempfile.gettempdir(), "test1")
        expected_file = join(utils.RESULTS_DIR, "test1")
        # Simulating a temp file (in the system tmp dir)
        shutil.copy(expected_file, temp_file)
        try:
            main._temp_to_file(temp_file, config_file)
            self.assertTrue(
                filecmp.cmp(config_file, expected_file, shallow=False))
        finally:
            if isfile(temp_file):
                os.remove(temp_file)

    def test_move_with_backup(self):
        """App must make a backup of the current config when generating a
        new one
        """
        config_file = join(utils.TMP_DIR, "test1")
        temp_file = join(tempfile.gettempdir(), "test1")
        expected_file = join(utils.RESULTS_DIR, "test1")
        expected_file_bak = join(utils.RESULTS_DIR, "test2")
        shutil.copy(expected_file, temp_file)
        # Creating a previous config file
        shutil.copy(expected_file_bak, config_file)
        try:
            main._temp_to_file(temp_file, config_file)
            self.assertTrue(
                filecmp.cmp(config_file, expected_file, shallow=False))
            self.assertTrue(
                filecmp.cmp(
                    "{0}.bak".format(config_file), expected_file_bak,
                    shallow=False))
        finally:
            if isfile(temp_file):
                os.remove(temp_file)


if __name__ == "__main__":
    unittest.main()



""" Update badge uuid cache
"""

import os
import time
from urllib.parse import parse_qs, urlparse

import boto3


# Contants
UUID_CACHE_TTL = 3600


def lambda_handler(event, context):
    update_uuid_cache()


def update_uuid_cache():
    codebuild_client = boto3.client("codebuild") 
    dynamodb_client = boto3.client("dynamodb")

    # Get current uuids
    uuids = []
    project_names = codebuild_client.list_projects()["projects"]
    for project in codebuild_client.batch_get_projects(names=project_names)["projects"]:
        if project["badge"]["badgeEnabled"]:
            uuid = parse_qs(urlparse(project["badge"]["badgeRequestUrl"]).query)["uuid"][0]
            uuids.append(uuid)

    # Update cache
    for uuid in uuids:
        dynamodb_client.put_item(
            TableName=os.environ["DYNAMODB_TABLE"],
            Item={
                "uuid": {"S": uuid},
                # Set a TTL so disabled build badges or deleted build projects will eventually
                # have their uuids deleted.
                "ttl": {"N": str(int(time.time()) + UUID_CACHE_TTL)}
            }
        )




import sys
import os
from os.path import join
from io import StringIO

import unittest

from update_conf_py_do_not_use import main
from . import utils


class ParseAllTest(unittest.TestCase):
    """Tests of command line parsing and config parsing
    """

    def setUp(self):
        self.file_path = join(utils.TMP_DIR, "test1")
        self.snippets_path = join(utils.SNIPPETS_DIR, "test1")
        self.config_path = utils.CONF_FILE
        self.section_name = "test1"
        self.config_file_path = "tests/tmp/test1"
        self.config_dir_path = "tests/tmp/test1.d"
        self.section_name_2 = "test2"
        self.config_file_path_2 = "tests/tmp/test2"
        self.config_dir_path_2 = "tests/snippets/test2"
        self.config_comment_prefix_2 = ";"
        # These tests must be done from the root dir
        self.chdir_old = os.getcwd()
        os.chdir(utils.ROOT_DIR)

    def tearDown(self):
        os.chdir(self.chdir_old)

    def test_short_cmd_args(self):
        """App must sucessfully parse cmd args (short options)
        """
        # Mock 'argv' with the new cmd args
        sys.argv = [
            sys.argv[0], "-f", self.file_path, "-d", self.snippets_path,
            "-p", "$"]
        args = main._parse_all()
        self.assertEqual(args.file, self.file_path)
        self.assertEqual(args.dir, self.snippets_path)
        self.assertEqual(args.comment_prefix, "$")

    def test_long_cmd_args(self):
        """App must sucessfully parse cmd args (full options)
        """
        sys.argv = [
            sys.argv[0], "--file", self.file_path, "--dir", self.snippets_path,
            "--comment-prefix", "$"]
        args = main._parse_all()
        self.assertEqual(args.file, self.file_path)
        self.assertEqual(args.dir, self.snippets_path)
        self.assertEqual(args.comment_prefix, "$")

    def test_default_parse(self):
        """App must set the correct default options

        It must set the default dir if no one is provided too.
        """
        sys.argv = [
            sys.argv[0], "--file", self.file_path]
        args = main._parse_all()
        self.assertEqual(args.file, self.file_path)
        self.assertEqual(args.dir, "{0}.d".format(self.file_path))
        self.assertEqual(args.comment_prefix, "#")
        self.assertEqual(args.config, [main.SYSTEM_CONFIG, main.USER_CONFIG])

    def test_wrong_cmd_args(self):
        """App must print an error and exit on wrong cmd args
        """
        sys.argv = [
            sys.argv[0], "--dir", self.snippets_path,
            "--comment-prefix", "$"]
        stderr_old, sys.stderr = sys.stderr, StringIO()
        try:
            with self.assertRaises(SystemExit):
                main._parse_all()
            output = sys.stderr.getvalue()
            self.assertTrue("file" in output and "required" in output)
        finally:
            sys.stderr = stderr_old

    def test_config_parse(self):
        """App must parse options from a config file (and use defaut values
        from the other options)
        """
        sys.argv = [
            sys.argv[0], "-c", self.config_path, "-n", self.section_name]
        args = main._parse_all()
        self.assertEqual(args.file, self.config_file_path)
        self.assertEqual(args.dir, self.config_dir_path)
        self.assertEqual(args.comment_prefix, "#")
        self.assertEqual(args.config, self.config_path)
        self.assertEqual(args.name, self.section_name)

    def test_config_parse_full(self):
        """App must parse all options from a config file
        """
        sys.argv = [
            sys.argv[0], "-c", self.config_path, "-n", self.section_name_2]
        args = main._parse_all()
        self.assertEqual(args.file, self.config_file_path_2)
        self.assertEqual(args.dir, self.config_dir_path_2)
        self.assertEqual(args.comment_prefix, self.config_comment_prefix_2)
        self.assertEqual(args.config, self.config_path)
        self.assertEqual(args.name, self.section_name_2)

    def test_nonexistent_config_parse(self):
        """App must print an error and exit when no config file was found
        """
        sys.argv = [sys.argv[0], "-c", "/non-existent", "-n", "test1"]
        stderr_old, sys.stderr = sys.stderr, StringIO()
        try:
            with self.assertRaises(SystemExit):
                main._parse_all()
            output = sys.stderr.getvalue()
            self.assertTrue("config file '/non-existent' not found" in output)
        finally:
            sys.stderr = stderr_old

    def test_wrong_config_parse(self):
        """App must print an error and exit on wrong config args
        """
        sys.argv = [sys.argv[0], "-c", self.config_path, "-n", "non_existent"]
        stderr_old, sys.stderr = sys.stderr, StringIO()
        try:
            with self.assertRaises(SystemExit):
                main._parse_all()
            output = sys.stderr.getvalue()
            self.assertTrue("section name 'non_existent' not found" in output)
        finally:
            sys.stderr = stderr_old

    def test_config_and_args_parse(self):
        """App must parse options from config/cmd args

        It must consider that cmd args takes precedence too.
        """
        sys.argv = [
            sys.argv[0], "-c", self.config_path, "-n", "test1",
            "-d", self.snippets_path]
        args = main._parse_all()
        self.assertEqual(args.file, self.config_file_path)
        self.assertEqual(args.dir, self.snippets_path)
        self.assertEqual(args.comment_prefix, "#")
        self.assertEqual(args.config, self.config_path)
        self.assertEqual(args.name, self.section_name)


if __name__ == "__main__":
    unittest.main()



import sys
from os.path import join, basename
# Use 'io.StringIO' for Python 3 compatibility. In Python 2, still use
# 'StringIO.StringIO' to avoid unicode errors.
try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

import unittest

from update_conf_py_do_not_use import main
from . import utils


class GetSnippetsTest(unittest.TestCase):
    """Tests to verify if the '_get_snippets' function works
    """

    def setUp(self):
        utils.clean_tmp()

    def tearDown(self):
        main.VERBOSE = False

    def test_snippets(self):
        """App must return all snippets in the correct order
        """
        the_dir = join(utils.SNIPPETS_DIR, "test1")
        files = main._get_snippets(the_dir)
        files = [basename(f) for f in files]
        expected_files = [
            "00-conf_0", "01-conf_1", "02-conf_2", "99-conf_99",
            "other_name-conf", "some_name-conf"]
        self.assertEqual(files, expected_files)

    def test_snippets_skip(self):
        """App must skip all invalid snippet filenames
        """
        the_dir = join(utils.SNIPPETS_DIR, "test1_2")
        files = main._get_snippets(the_dir)
        files = [basename(f) for f in files]
        expected_files = [
            "00-conf_0", "01-conf_1", "02-conf_2", "99-conf_99",
            "other_name-conf", "some_name-conf"]
        self.assertEqual(files, expected_files)

    def test_snippets_wrong_dir(self):
        """App must print an error and exist if wrong dirs are used

        Empty dirs and non-existend dirs are the two tested cases.
        """
        # Non-existent dir
        stderr_old, sys.stderr = sys.stderr, StringIO()
        try:
            with self.assertRaises(SystemExit):
                main._get_snippets("/non-existent")
            output = sys.stderr.getvalue()
            self.assertTrue("dir" in output and "not found" in output)
        finally:
            sys.stderr = stderr_old
        # Empty dir
        stderr_old, sys.stderr = sys.stderr, StringIO()
        try:
            with self.assertRaises(SystemExit):
                main._get_snippets(utils.TMP_DIR)
            output = sys.stderr.getvalue()
            self.assertTrue("no snippet" in output and "found" in output)
        finally:
            sys.stderr = stderr_old

    def test_snippets_verbose(self):
        """App must print verbose messages about skips when verbosity is
        active
        """
        main.VERBOSE = True
        the_dir = join(utils.SNIPPETS_DIR, "test1_2")
        stdout_old, sys.stdout = sys.stdout, StringIO()
        try:
            main._get_snippets(the_dir)
            output = sys.stdout.getvalue()
            self.assertTrue("Skipping" in output and "01-conf_1.bak" in output)
            self.assertTrue(
                "Skipping" in output and "03-conf_3.disabled" in output)
        finally:
            sys.stdout = stdout_old


if __name__ == "__main__":
    unittest.main()



import sys
import os
from os.path import join
import subprocess
import filecmp

import unittest

from update_conf_py_do_not_use import main
from . import utils


class ScriptTest(unittest.TestCase):
    """Tests that use the final script
    """

    def setUp(self):
        utils.clean_tmp()
        # These tests must be done from the root dir
        self.chdir_old = os.getcwd()
        os.chdir(utils.ROOT_DIR)

    def tearDown(self):
        os.chdir(self.chdir_old)

    def test_script_cmd(self):
        """Script must merge files using cmd args
        """
        file_path = join(utils.TMP_DIR, "test1")
        expected_path = join(utils.RESULTS_DIR, "test1")
        dir_path = join(utils.SNIPPETS_DIR, "test1_2")
        args = [utils.APP]
        args += ["-f", file_path, "-d", dir_path]
        subprocess.call(args)
        # The second call forces a backup
        subprocess.call(args)
        self.assertTrue(
            filecmp.cmp(file_path, expected_path, shallow=False))
        self.assertTrue(filecmp.cmp(
            "{0}.bak".format(file_path), expected_path, shallow=False))

    def test_script_config(self):
        """Script must merge files using args from config
        """
        file_path = join(utils.TMP_DIR, "test2")
        section_name = "test2"
        expected_path = join(utils.RESULTS_DIR, "test2")
        args = [utils.APP]
        args += ["-c", utils.CONF_FILE, "-n", section_name]
        subprocess.call(args)
        # The second call forces a backup
        subprocess.call(args)
        self.assertTrue(
            filecmp.cmp(file_path, expected_path, shallow=False))
        self.assertTrue(filecmp.cmp(
            "{0}.bak".format(file_path), expected_path, shallow=False))

    def test_script_error(self):
        """Script must print an error and exit if something wrong occur

        Only some tests are done, because most of them are already tested
        in the test cases of inner level functions.
        """
        file_path = join(utils.TMP_DIR, "test1")
        dir_path = "/non-existent"
        args = [utils.APP]
        args += ["-f", file_path, "-d", dir_path]
        with self.assertRaises(subprocess.CalledProcessError):
            subprocess.check_call(
                args,
                stderr=subprocess.DEVNULL
            )

    def test_script_verbose(self):
        """Script must print verbose messages when verbose is set
        """
        file_path = join(utils.TMP_DIR, "test1")
        dir_path = join(utils.SNIPPETS_DIR, "test1_2")
        args = [utils.APP]
        args += ["-f", file_path, "-d", dir_path]
        subprocess.call(args)
        # Set verbose flag
        args += ["-v"]
        output = str(subprocess.check_output(args))
        self.assertTrue(
            "Skipping" in output and "Merging" in output
            and "Backing up" in output)

    def test_run(self):
        """Test a call to the run function

        Unfortunately, the previous tests from this module are not detected
        by 'coverage'. We are testing this function directaly (the test is
        equal to the 'test_script_cmd'). So, 'coverage' will consider that the
        'run' function is tested.
        """
        file_path = join(utils.TMP_DIR, "test1")
        expected_path = join(utils.RESULTS_DIR, "test1")
        dir_path = join(utils.SNIPPETS_DIR, "test1_2")
        args = [utils.APP]
        args += ["-f", file_path, "-d", dir_path]
        # Mock sys.argv
        self.argv_old = sys.argv
        sys.argv = args
        main.run()
        # The second call forces a backup
        main.run()
        self.assertTrue(
            filecmp.cmp(file_path, expected_path, shallow=False))
        self.assertTrue(filecmp.cmp(
            "{0}.bak".format(file_path), expected_path, shallow=False))
        sys.argv = self.argv_old


if __name__ == "__main__":
    unittest.main()



""" Return CodeBuild build status information for API integration with
shields.io

It works standalone (with Lambda HTTP endpoints) or via integration with
API Gateway.
"""

import json
import os

import boto3


def lambda_handler(event, context):
    # Validate requests from Lambda HTTPS endpoint
    if "http" in event["requestContext"]:
        if event["requestContext"]["http"]["method"] != "GET":
            return { "statusCode": 405 }
        if event["requestContext"]["http"]["path"] != "/build-status":
            return { "statusCode": 404 }
        if not "build_project" in event["queryStringParameters"]:
            return { 
                "statusCode": 400,
                "body": json.dumps({
                    "message": "Missing required request parameters: [build-project]"
                })
            }
    # Get "build-project" for Lambda HTTPS endpoint or API Gateway 
    # integration
    build_project = event["queryStringParameters"]["build-project"]

    return {
        "statusCode": 200,
        "body": get_shields_io_json(build_project)
    }


def get_shields_io_json(build_project):
    dynamodb_client = boto3.client("dynamodb")
    
    # Get build_status
    response = dynamodb_client.get_item(
        TableName=os.environ["DYNAMODB_TABLE"],
        Key={"build_project": {"S": build_project}}
    )
    try:
        build_status = response["Item"]["build_status"]["S"]
    except KeyError:
        build_status = None

    # Map build status to message/color
    if build_status == "SUCCEEDED":
        message = "passing"
        color = "success"
    elif build_status == "FAILED":
        message = "failing"
        color = "critical"
    elif not build_status:
        message = "no status"
        color = "inactive"
    else:
        raise RuntimeError("Not expected `build_status`: {}".format(build_status))
    
    # Return JSON in the format expected by shields.io
    # See: https://shields.io/endpoint
    return json.dumps({
        "schemaVersion": 1,
        "label": build_project,
        "message": message,
        "color": color
    }, indent=4)




"""update-conf-DO-NOT-USE setup.py file
"""

import os
from os.path import abspath, dirname, join
from setuptools import setup

from update_conf_py_do_not_use import main


# Consts
GITHUB_URL = "https://us-east-1.console.aws.amazon.com/codesuite/codecommit/repositories/update-conf-py-DO-NOT-USE/"
README = "README.md"

# Important vars
cur_dir = abspath(dirname(__file__))
readme = join(cur_dir, README)
sample_config = join("samples", main.CONFIG_NAME)
# Get description from the first line of the module docstring.
description = main.__doc__.split('\n')[0]
long_description = ""
with open(readme, 'r') as f:
    long_description = f.read()


# Setup
setup(
    # Main software info
    name=main.__program__,
    version=main.__version__,
    description=description,
    long_description=long_description,
    long_description_content_type="text/markdown",
    license=main.__license__,
    author=main.__author__,
    author_email=main.__email__,
    url=GITHUB_URL,
    keywords="system unix config split merge snippets sysadmin",
    packages=["update_conf_py_do_not_use"],

    # Requirements
    python_requires=">=3.7",

    # Classifiers
    # See: https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        "Development Status :: 1 - Planning",
        "Environment :: Console",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved",
        "Natural Language :: English",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: POSIX",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Topic :: System :: Systems Administration",
        "Topic :: Utilities",
    ],

    # Entry points
    # The script itself is defined here.
    entry_points={
        "console_scripts": [
            "update-conf-py-do-not-use = update_conf_py_do_not_use:run",
        ],
    },

    # Data files
    data_files=[
        (join("share", main.__program__), [sample_config, ]),
    ],

    # Tests
    test_suite="tests",
)



import sys
import os
from os.path import join, isfile
import shutil
from io import StringIO

import unittest

from update_conf_py_do_not_use import main
from . import utils


@unittest.skipIf(
    os.geteuid() != 0, "supported only with root (or sudo)")
class SudoConfigTest(unittest.TestCase):
    """Tests for the config read feature, but that requires root privileges

    This tests SHOULD be run only in DEVELOPMENT envs. There are risks of
    destroying your original config files. Keep CAUTION!
    """

    def setUp(self):
        self.section_name = "test1"
        self.expected_file_global = "tests/tmp/test1"
        self.expected_file_user = "/nonexistent/home"
        self.backup_global = join(utils.TMP_DIR, "config_global.bak")
        self.backup_user = join(utils.TMP_DIR, "config_user.bak")
        utils.clean_tmp()
        # Mock 'argv' with the new cmd args
        self.argv_old = sys.argv
        sys.argv = [sys.argv[0], "-n", self.section_name]
        # Move/backup original config files
        if isfile(main.SYSTEM_CONFIG):
            shutil.move(main.SYSTEM_CONFIG, self.backup_global)
        if isfile(main.USER_CONFIG):
            shutil.move(main.USER_CONFIG, self.backup_user)

    def tearDown(self):
        sys.argv = self.argv_old
        # Clean config files. If they exist, they are test files.
        if isfile(main.SYSTEM_CONFIG):
            os.remove(main.SYSTEM_CONFIG)
        if isfile(main.USER_CONFIG):
            os.remove(main.USER_CONFIG)
        # Restore original config files
        if isfile(self.backup_global):
            shutil.move(self.backup_global, main.SYSTEM_CONFIG)
        if isfile(self.backup_user):
            shutil.move(self.backup_user, main.USER_CONFIG)

    def test_system_config(self):
        """App must read config file from system etc if it exits
        """
        shutil.copy(utils.CONF_FILE, main.SYSTEM_CONFIG)
        args = main._parse_all()
        self.assertEqual(args.file, self.expected_file_global)

    def test_user_config(self):
        """App must read config file from user home if it exits
        """
        shutil.copy(utils.USER_CONF_FILE, main.USER_CONFIG)
        args = main._parse_all()
        self.assertEqual(args.file, self.expected_file_user)

    def test_system_user_config(self):
        """Options from user config file MUST take precedence over the
        system one
        """
        shutil.copy(utils.CONF_FILE, main.SYSTEM_CONFIG)
        shutil.copy(utils.USER_CONF_FILE, main.USER_CONFIG)
        args = main._parse_all()
        self.assertEqual(args.file, self.expected_file_user)

    def test_default_config_not_found(self):
        """App must print a error if none of the default config files were
        found
        """
        stderr_old, sys.stderr = sys.stderr, StringIO()
        try:
            with self.assertRaises(SystemExit):
                main._parse_all()
            output = sys.stderr.getvalue()
            self.assertTrue(
                "neither" in output and "nor" in output
                and main.SYSTEM_CONFIG in output
                and main.USER_CONFIG in output)
        finally:
            sys.stderr = stderr_old


if __name__ == "__main__":
    unittest.main()



import sys
import os
from os.path import join
import filecmp
from io import StringIO

import unittest

from update_conf_py_do_not_use import main
from . import utils


class CreateTempConfigTest(unittest.TestCase):
    """Tests of creating a new config temp file

    This tests the autogenerated comments too.
    """

    def setUp(self):
        self.snippets_1 = []
        self.snippets_2 = []

        the_dir = join(utils.SNIPPETS_DIR, "test1")
        snippets = [
            "00-conf_0", "01-conf_1", "02-conf_2", "99-conf_99",
            "other_name-conf", "some_name-conf"]
        self.snippets_1 = [join(the_dir, s) for s in snippets]
        the_dir = join(utils.SNIPPETS_DIR, "test2")
        snippets = ["00-conf_0", "some_name-conf"]
        self.snippets_2 = [join(the_dir, s) for s in snippets]

    def tearDown(self):
        main.VERBOSE = False

    def test_tmpconfig_1(self):
        """App must merge sucessfully all snippets from test1

        It must to add the correct autogenerated message too.
        """
        temp_file = main._create_temp_config(
            self.snippets_1, comment_prefix="#")
        try:
            self.assertTrue(filecmp.cmp(
                temp_file, join(utils.RESULTS_DIR, "test1"), shallow=False))
        finally:
            os.remove(temp_file)

    def test_tmpconfig_2(self):
        """App must merge sucessfully all snippets from test2

        It must to add the correct autogenerated message too.
        """
        temp_file = main._create_temp_config(
            self.snippets_2, comment_prefix=";")
        try:
            self.assertTrue(filecmp.cmp(
                temp_file, join(utils.RESULTS_DIR, "test2"), shallow=False))
        finally:
            os.remove(temp_file)

    def test_tmpconfig_verbose(self):
        """App must print verbose messages about merged files
        """
        main.VERBOSE = True
        stdout_old, sys.stdout = sys.stdout, StringIO()
        try:
            temp_file = main._create_temp_config(
                self.snippets_2, comment_prefix=";")
            output = sys.stdout.getvalue()
            self.assertTrue("Merging" in output and "00-conf_0" in output)
            self.assertTrue("Merging" in output and "some_name-conf" in output)
        finally:
            sys.stdout = stdout_old
            os.remove(temp_file)


if __name__ == "__main__":
    unittest.main()
